package ids;

import java.util.regex.*;
import javax.swing.*;

public
class SnortSignature extends AnySignature{

    public String sig_source; // shows wheather the signature comes from snort or dragon
    public String protected_network; 	// protected network ip

    public String description = "";	// signature description (cve references, and others)
    public String action;			// what should be done after
    public String protocol;			// which protocol

    public String source_ip;		// source ip numbers
    public boolean source_ip_neg;		// if true then negate source_ips
    public String source_port;		// source port number of the packet
    public boolean source_port_neg;		// if true then negate source_port
    public String destination_ip;		// destination ip numbers
    public boolean destination_ip_neg;	// if true then negate destination_ips
    public String destination_port;		// destination port of the packet
    public boolean destination_port_neg;	// if true then negate destination_port
    public String direction;		// $External -> $Protected, Protected -> External, External <> Protected

    public String pattern;			// text, we are looking for
    public String pattern_type;		// is it this text EXACTLY or something like this
    public boolean pattern_neg;		// catch packet only with this or only without this text
    public String message;			// what mesage text should be send to You after alarm

    public String comment = "";			// additional comments for rule

    public int loged_packets; // number of packet logged after alarm occurs^M
    public int search_depth = 0;		// how many bytes each packet should be checked
    public boolean success;
    public String result = "";

    public static String default_action = "alert";


        public char switchCase(String expr, String[] args) {
              if (args.length % 2 != 0)
                    return 0;
              for (int i = 0; i < args.length; i += 2)
                    if (expr.equals(args[i]))
                          return args[i + 1].charAt(0);
              return 0;
        }

    public SnortSignature(CommonSignature cs){
        protected_network = cs.protected_network;	// protected network address
        description = cs.description;				// description of the signature

        action = (cs.action!="") ? cs.action : default_action;
        protocol = cs.protocol;						// protocol
        if(cs.source_port == null || cs.source_port.equals("") || cs.source_port.equals("any"))	// ports
            source_port = "any";
        else
            source_port = cs.source_port;
        source_port_neg = cs.source_port_neg;
        if(cs.source_port == null || cs.destination_port.equals("") || cs.destination_port.equals(""))
            destination_port = "any";
        else
            destination_port = cs.destination_port;
        destination_port_neg = cs.destination_port_neg;
        source_ip = cs.source_ip;
        destination_ip = cs.destination_ip;
        source_ip_neg = cs.source_ip_neg;
        destination_ip_neg = cs.destination_ip_neg;
        if(source_ip.equals("any")&&destination_ip.equals("any") && source_port.equals("any")&&destination_port.equals("any"))
            direction = "<>";
        else
            direction = "->";
        message = cs.message;
        pattern = cs.pattern;
        pattern_type = cs.pattern_type;
        pattern_neg = cs.pattern_neg;
        search_depth = cs.search_depth;
        result = "Snort constructor: CIDSS -> Snort SUCCEED";
        success = true;
    }


    
    public SnortSignature(String str){
        result = "";
        success = false;

		Pattern p;
		Matcher m;
		String s = str;

		String sAction = "alert|log|pass|activate|dynamic";
		String sProto  = "tcp|udp|igmp|ip";
		String negation = "!?"; 	// negation one or nothing
		// [ip/mask]; ip; ip/mask
		String sIP = "\\[.*{1}\\]|\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}|\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}/\\d{1,2}";
		String sIPname = "\\$\\S+|any";		// string (environment variable) starting with $ or "any"
		String sPort = "any|\\d{1,5}|\\$\\S+";			// port number - number contains 1-5 digits
		String sDirection = "->|<>";

		p = Pattern.compile("^("+sAction+")\\s?(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			action = m.group(1);
			s = m.group(2);
		} else {
			result = "Error - unknown action";
			showError(result);
			return;
		}

		p = Pattern.compile("^("+sProto+") (.*)");
		m = p.matcher(s);
		if(m.matches()) {
			protocol = m.group(1);
			s = m.group(2);
		} else {
			result = "Error - unknown protocol";
			showError(result);
			return;
		}

		p = Pattern.compile("^("+negation+")\\s?(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			source_ip_neg = m.group(1).equals("!")?true:false;
			s = m.group(2);
		} else {
			result = "Error - unknown source host negation";
			showError(result);
			return;
		}
		p = Pattern.compile("^("+sIP+"|"+sIPname+") (.*)");
		m = p.matcher(s);
		if(m.matches()) {
			source_ip = m.group(1);
			s = m.group(2);
		} else {
			result = "Error - unknown source host";
			showError(result);
			return;
		}

		p = Pattern.compile("^("+sPort+") (.*)");
		m = p.matcher(s);
		if(m.matches()) {
			source_port = m.group(1);
			s = m.group(2);
		} else {
			result = "Error - unknown source port";
			showError(result);
			return;
		}

		p = Pattern.compile("^("+sDirection+") (.*)");
		m = p.matcher(s);
		if(m.matches()) {
			direction = m.group(1);
			s = m.group(2);
		} else {
			result = "Error - unknown rule direction";
			showError(result);
			return;
		}

		p = Pattern.compile("^("+negation+")\\s?(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			destination_ip_neg = m.group(1).equals("!")?true:false;
			s = m.group(2);
		} else {
			result = "Error - unknown destination host negation";
			showError(result);
			return;
		}
		p = Pattern.compile("^("+sIP+"|"+sIPname+") (.*)");
		m = p.matcher(s);
		if(m.matches()) {
			destination_ip = m.group(1);
			s = m.group(2);
		} else {
			result = "Error - unknown destination host";
			showError(result);
			return;
		}

		p = Pattern.compile("^("+sPort+") (.*)");
		m = p.matcher(s);
		if(m.matches()) {
			destination_port = m.group(1);
			s = m.group(2);
		} else {
			result = "Error - unknown destination port";
			showError(result);
			return;
		}

		// Snort MetaData
		String sMsg = "msg:\\s?\"(.*?)\";";
		String sClasstype = "classtype:\\s?.*?;";
		String sSid = "sid:\\s?.*?;";
		String sRev = "rev:\\s?.*?;";
		String sPriority = "priority:\\s?.*?;";
		
		p = Pattern.compile("\\((.*)\\).*?"); // cut off brackets
		m = p.matcher(s);
		if(m.matches()) {
			s = m.group(1);
		} else {
			result = "Error - unknown schema - no brackets in rule!";
			showError(result);
			return;
		}

		p = Pattern.compile("(.*?)("+sMsg+")(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			message = m.group(3);
			s = m.group(1)+m.group(4);
		} else {
			result = "Error - unknown message";
			showError(result);
			return;
		}

		p = Pattern.compile("(.*?)("+sClasstype+")(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			s = m.group(1)+m.group(3);
		} else {
			// classtype is not required; continue
		}

		p = Pattern.compile("(.*?)("+sSid+")(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			description = m.group(2);
			s = m.group(1)+m.group(3);
		} else {
			// sid is not required; continue
		}

		p = Pattern.compile("(.*?)("+sRev+")(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			s = m.group(1)+m.group(3);
		} else {
			// rev is not required; continue
		}

		p = Pattern.compile("(.*?)("+sPriority+")(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			s = m.group(1)+m.group(3);
		} else {
			// priority is not required; continue
		}

		// Snort MetaData Content
		String sContent = "content:\\s*("+negation+")\\s*\"(.*?)\";";
		String sDepth = "depth:\\s*(\\d+);";

		p = Pattern.compile("(.*)("+sContent+")(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			pattern_neg = m.group(3).equals("!")?true:false;
			pattern = m.group(4);
			pattern_type = "string";
			s = m.group(1)+m.group(5);
		} else {
			result = "Error - unknown content";
			showError(result);
			return;
		}

		p = Pattern.compile("(.*?)("+sContent+")(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			result = "Error - more than 1 content found! "+
				"Assuming first occurrence as correct content";
//			showError(result);return;
		}
		
		p = Pattern.compile("(.*)("+sDepth+")(.*)");
		m = p.matcher(s);
		if(m.matches()) {
			search_depth = Integer.parseInt(m.group(3));
			s = m.group(1)+m.group(4);
		} else {
//			System.out.println("Error - unknown search depth - assuming 0...");
			search_depth = 0;
//			showError(result);return;
		}

		message = replaceAll(message,"/20"," ");
		pattern = replaceAll(pattern,"/20"," ");

        result = "Snort constructor(str): SUCCEED";
        success = true;

    }


    /* fix this */
    public CommonSignature toCommon(){

        result = "";

        boolean sourc_ip_neg = false;
        boolean dest_ip_neg = false;
        boolean s_port = false;
        boolean d_port = false;


        result = "Snort, toCommon(): SUCCEED";
        success = true;
        return new CommonSignature(action, protocol, "Snort", description,
                                    source_ip, sourc_ip_neg, destination_ip, dest_ip_neg,
                                    source_port,  s_port, destination_port, d_port,
                                    pattern, pattern_type, pattern_neg, message,
                                    0, search_depth, comment);

    }

    public String getDescribe(){
        String s = "";
        s += "\n----------------------------------------------------\n";
        s += "\nCIDSS Signature:\n";
        s += "\naction: " + action;
        s += "\nprotocol: " + protocol;
        s += "\nsignature source: " + sig_source;
        s += "\ndescription: " + description;

        s += "\nSource IP: " + source_ip;
        s += "\nSource IP negation: " + source_ip_neg;
        s += "\nSource port: " + source_port;
        s += "\nSource port negation: " + source_port_neg;

        s += "\nDestination IP: " + destination_ip;
        s += "\nDestination IP negation: " + destination_ip_neg;
        s += "\nDestination port: " + destination_port;
        s += "\nDestination port negation: " + destination_port_neg;

        s += "\nType of text we are looking for: " + pattern_type;
        s += "\nText we are looking for: " + pattern;
        s += "\nText negation?: " + pattern_neg;
        s += "\nMessage send after alarm: " + message;
        s += "\nNumber of packets logged: " + loged_packets;
        s += "\nSearch depth: " + search_depth;
        s += "\nAdditional informations: " + comment;
        s += "\n------------------------------------------------------\n";

        return s;
    }



    public String getSignature(){

        String string = "";

        string += action + " ";
        string += protocol + " ";
        string += source_ip_neg?"!":"";
        string += source_ip + " ";
        string += source_port_neg?"!":"";
        string += source_port + " ";
        string += direction + " ";
        string += destination_ip_neg?"!":"";
        string += destination_ip + " ";
        string += destination_port_neg?"!":"";
        string += destination_port + " ";
        //-------------------------------------
        string += "(msg:\"" + message + "\"; ";
        String tmp[] = pattern.split(" ");
        for(int i=0; i<tmp.length; i++){
            string += "content: \"" + tmp[i] + "\";";
        }
        if(search_depth != 0)
            string += "depth:" + search_depth + ";";

        string += ")";
        return string;
    }

	private void showError(String err) {
		JOptionPane.showMessageDialog(null,err,"Snort signature error",
			JOptionPane.ERROR_MESSAGE);
	}
}
